import 'dart:math';
class AxlLatLng {
  final double latitude;
  final double longitude;

  AxlLatLng(this.latitude, this.longitude);

  bool isEqual(AxlLatLng that) {
    return (this.latitude == that.latitude && this.longitude == that.longitude);
  }

  int toX(int zoom) {
    return ((this.longitude + 180) / 360 * pow(2, zoom)).floor();
  }

  int toY(int zoom) {
     return ((1 - log(tan(this.latitude * pi / 180) + 1 / cos(this.latitude * pi / 180)) / pi) /
            2 *
            pow(2, zoom))
        .floor();
  }

  List<String> tilesAround() {
    List<String> res = List<String>();
    ["12","13","14","15","16"].forEach((zoom) {
      int x = this.toX(int.parse(zoom));
      int y = this.toY(int.parse(zoom));
      res.add(tile(x-2, y, zoom));
      res.add(tile(x-1, y, zoom));
      res.add(tile(x, y, zoom));
      res.add(tile(x, y-2, zoom));
      res.add(tile(x, y-1, zoom));
    });
  }

  String tile(int x,int y, String z) {
    return '{$x,$y,$z}';
  }

  double dist2(AxlLatLng that) =>
    (latitude - that.latitude) * (latitude - that.latitude) + (longitude - that.longitude) * (longitude - that.longitude);

  int orientation(AxlLatLng p1, AxlLatLng p2) { 
    double val = (longitude - p1.longitude) * (p2.latitude - latitude) - 
              (latitude - p1.latitude) * (p2.longitude - longitude); 
    return val.compareTo(0);
  }

  static List<AxlLatLng> convexHull(List<AxlLatLng> points) {
    if (points.length < 4) return points;
    // Find bottom
    AxlLatLng bottom = points[0];
    for (var p in points) {
      if (p.latitude < bottom.latitude) bottom = p;
      else if (p.latitude == bottom.latitude && p.longitude < bottom.longitude) bottom = p;
    }
    List<AxlLatLng> others = points.where((p) => !bottom.isEqual(p)).toList();
    // sort
    others.sort((p1, p2) => bottom.orientation(p1, p2));
    
    if (others.length < 3) return [bottom] + others;
    int top = 0;
    int ind = 1;
    // remove same point
    while (ind < others.length) {
      if (!others[ind].isEqual(others[top]) && bottom.orientation(others[top], others[ind]) != 0) {
        top++;
        others[top] = others[ind];
      }
      ind++;
    }
    others = others.sublist(0, top);
    if (top < 3) return [bottom] + others;
    top = 1;
    ind = 2;
    while (ind < others.length) {
      while (top > 0 && others[top - 1].orientation(others[top], others[ind]) >= 0) top--;
      top++;
      others[top] = others[ind];
      ind++;
    }
    return [bottom] + others.sublist(0, top + 1);
  }
}
