import 'dart:typed_data';
import 'dart:ui';
import 'dart:async';
import 'package:flutter/material.dart';

import 'axl_latlng.dart';

@immutable
class AxlMarker {
  const AxlMarker({
        @required this.latLng,
        this.onSelected,
        this.width,
        this.height,
        this.anchorLeft,
        this.anchorTop,
        this.icon,
        this.text,
        this.textSize = 12,
        this.textColor = "#FFFFFF",
        this.textOffset,
        this.iconOffset,
        this.image = "images/mapPin.png"})
        : assert(latLng != null);

  final AxlLatLng latLng;
  final VoidCallback onSelected;
  final WidgetBuilder icon;
  final double width;
  final double height;
  final double anchorLeft;
  final double anchorTop;
  final String image;
  final String text;
  final double textSize;
  final String textColor;
  final Offset textOffset;
  final Offset iconOffset;


  static Future<Uint8List> getBytesFromCanvas(
      String text, int size,
      [Color markerColor = Colors.blue, Color textColor = Colors.white]
  ) async {
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder);
    final Paint paint = Paint()..color = markerColor;
    final Radius radius = Radius.circular(100.0);
    final double drawOffset = size/10;

    Path path = Path();
    path.moveTo(drawOffset*1.5, size/2 + drawOffset);
    path.lineTo(size/2, size.toDouble());
    path.lineTo(size - drawOffset*1.5, size/2 + drawOffset);
    path.close();

    canvas.drawPath(path, paint);

    canvas.drawRRect(
        RRect.fromRectAndCorners(
          Rect.fromLTWH(
              drawOffset, 0.0,
              size.toDouble() - drawOffset*2,
              size.toDouble() - drawOffset*2
          ),
          topLeft: radius,
          topRight: radius,
          bottomLeft: radius,
          bottomRight: radius,
        ),
        paint
    );

    TextPainter painter = TextPainter(textDirection: TextDirection.ltr);
    painter.text = TextSpan(
      text: text,
      style: TextStyle(fontSize: size/2, color: textColor),
    );
    painter.layout();
    painter.paint(
        canvas,
        Offset(
            (size * 0.5) - painter.width * 0.5,
            (size * 0.5) - painter.height * 0.5 - drawOffset)
    );
    final img = await pictureRecorder.endRecording().toImage(size, size);
    final data = await img.toByteData(format: ImageByteFormat.png);

    return data.buffer.asUint8List();
  }
}
